#include <stdio.h>
#include <math.h>   
#include "gba.h"

#include "font.h"

u16 *BgPal=(u16*)0x5000000;
u16 *tiledata=(u16*)CharBaseBlock(0);
u16 *tilemap=(u16*)ScreenBaseBlock(31);
u16 *tilemap2=(u16*)ScreenBaseBlock(30);
u16 *tilemap3=(u16*)ScreenBaseBlock(29);
u16* screen = (u16*)0x6000000;

u16* OAMMem = (u16*)0x7000000;				//Object Attribute Memory
u16* TiledSpriteMem = (u16*)0x6010000;		//Sprite memory with tiled background
u16* SpritePal = (u16*)0x5000200;

TypeSprite Sprites[128];
u32 clear = 0;
Pads Pad;
u16 CompletePad, ExPad, TempPad;

void bir_UpdatePad()
{
	ExPad = CompletePad;
  CompletePad = ~(BUTTONS);
  UPDATEPAD(Pad.Held, CompletePad);
	UPDATEPAD(Pad.Released, (ExPad & (~CompletePad)));
  UPDATEPAD(Pad.Newpress, (CompletePad & (~ExPad)));
}

void bir_UpdateOAM() 
{
	u16 i;
	u16* temp;
	temp = (u16*)Sprites;
	for(i=0; i < 128*4; i++)	{
		OAMMem[i] = temp[i];
	}
}

int pos_bgtile = 0;

void bir_InitText() 
{
	BgPal[1] = RGB(0,0,0);
	BgPal[2] = RGB(6,6,6);
	BgPal[3] = RGB(12,12,12);
	BgPal[4] = RGB(18,18,18);
  DMACopyCH3((void*)fontData, (void*)CharBaseBlock(0), fontDataSize32, DMA_32NOW);
  pos_bgtile += fontDataSize32;
}

void bir_LoadBgPal(u16* palette)
{
	DMACopyCH3((void*)palette, (void*)BgPal, 128, DMA_32NOW);
}

void bir_LoadBg(u8 bg, u16* data, int datasize,u16* map, int mapsize)
{
	DMACopyCH3((void*)data, (void*)(tiledata+pos_bgtile), datasize, DMA_32NOW);
	pos_bgtile += datasize;
	DMACopyCH3((void*)map, (void*)tilemap3, mapsize, DMA_32NOW);
}

void bir_LoadSpritePal(u16* palette)
{
	DMACopyCH3((void*)palette, (void*)SpritePal, 128, DMA_32NOW);
}

int tilenum = 0;
int pos_mem = 0;

int bir_CreateSpriteGfx(u16* data,u16 size)
{
	int tmp_tilenum = tilenum;
	switch(size)
	{
		case SIZE_8:
			DMACopyCH3((void*)data, (void*)(TiledSpriteMem+pos_mem), 32, DMA_32NOW);
			pos_mem += 32;
			tilenum += 2;
		break;
		
		case SIZE_16:
			DMACopyCH3((void*)data, (void*)(TiledSpriteMem+pos_mem), 128, DMA_32NOW);
			pos_mem += 256;
			tilenum += 16;
		break;
		
		case SIZE_32:
			DMACopyCH3((void*)data, (void*)(TiledSpriteMem+pos_mem), 512, DMA_32NOW);
			pos_mem += 1024;
			tilenum += 64;
		break;
		
		case SIZE_64:
			DMACopyCH3((void*)data, (void*)(TiledSpriteMem+pos_mem), 2048, DMA_32NOW);
			pos_mem += 4096;
			tilenum += 256;
		break;
		
		default:
			
		break;
	}
	
	return	tmp_tilenum;
}

void bir_CreateSpriteFromGfx(u8 n,int tilenum, u16 size, u8 x, u8 y)
{
	Sprites[n].attr0 = COLOR_256 | SQUARE |	y;
	Sprites[n].attr1 = size |	x;
	Sprites[n].attr2 = tilenum;
}

void bir_LoadSprite(u8 n, u16* data,u16 size, u8 x, u8 y)
{
	Sprites[n].attr0 = COLOR_256 | SQUARE |	y;
	Sprites[n].attr1 = size |	x;
	Sprites[n].attr2 = tilenum;
	
	switch(size)
	{
		case SIZE_8:
			DMACopyCH3((void*)data, (void*)(TiledSpriteMem+pos_mem), 32, DMA_32NOW);
			pos_mem += 32;
			tilenum += 2;
		break;
		
		case SIZE_16:
			DMACopyCH3((void*)data, (void*)(TiledSpriteMem+pos_mem), 128, DMA_32NOW);
			pos_mem += 256;
			tilenum += 16;
		break;
		
		case SIZE_32:
			DMACopyCH3((void*)data, (void*)(TiledSpriteMem+pos_mem), 512, DMA_32NOW);
			pos_mem += 1024;
			tilenum += 64;
		break;
		
		case SIZE_64:
			DMACopyCH3((void*)data, (void*)(TiledSpriteMem+pos_mem), 2048, DMA_32NOW);
			pos_mem += 4096;
			tilenum += 256;
		break;
		
		default:
			
		break;
	}
	
}

void bir_SetSpriteXY(u8 n, s16 x, s16 y)
{
	Sprites[n].attr0 = (Sprites[n].attr0&(65535-255)) + (y&255);
	Sprites[n].attr1 = (Sprites[n].attr1&(65535-511)) + (x&511);
}

void bir_SpriteInit()
{
	u16 i;
	for (i = 0; i < 128; i++)
	{
		Sprites[i].attr0 = 160;
		Sprites[i].attr2 = 240;
	}
}

void bir_Init(int mode) 
{
	switch(mode) 
	{
		case MODE_0:
			REG_DISP_CNT = VIDEO_MODE0 | VIDEO_BG0_ENABLED | VIDEO_BG1_ENABLED | OBJ_ENABLED | OBJ_MAP_1D;

			REG_BG0CNT = BG_TEXTE_256x256 | BG_WRAP | BG_1_PALETTE | BG_CHARBLOCK_0 | 31<<8;
			REG_BG1CNT = BG_TEXTE_256x256 | BG_WRAP | BG_1_PALETTE | BG_CHARBLOCK_1 | 30<<8;
			REG_BG2CNT = BG_TEXTE_256x256 | BG_WRAP | BG_1_PALETTE | BG_CHARBLOCK_2 | 29<<8;
			
			BgPal[0] = RGB(31,31,31);
			
			bir_SpriteInit();
		break;
		
		case MODE_1:
			REG_DISP_CNT = VIDEO_MODE1;
		break;
		
		case MODE_4:
			REG_DISP_CNT = 0x0404;
		break;
		
		default:
			
		break;
		
	}
}


void bir_Print(char* str, u8 x, u8 y) {
  int i = 0;
  u16 *map=(u16*)ScreenBaseBlock(31);
  while(str[i]) 
  {
  	map[x+y*32+ i] = str[i];
  	i++;
  }
}

void bir_WaitVBL() 
{
	bir_UpdateOAM();
	bir_UpdatePad();
	while(REG_VCOUNT != 160);
}

void bir_ScrollBgXY(u8 bg, u8 x, u8 y) 
{
	REG_BGSCROLLX(bg&3) = x;
	REG_BGSCROLLY(bg&3) = y;
}

void bir_Put8bitPx(u8 x, u8 y, u8 color) 
{
	screen[(x>>1)+y*120] &= 0xFF00>>((x&1)<<3);
  screen[(x>>1)+y*120] |= color<<((x&1)<<3);
}

void bir_ClearScreen()
{
	DMACopyCH3(0, (void*)screen, 240*40, DMA_32NOW);
}



void inline DMACopyCH0(void* source, void* dest, u32 WordCount, u32 mode) {
    REG_DMA0SAD = (u32)source;      //Tell the gba our source address
    REG_DMA0DAD = (u32)dest;        //Tell the gba where the data should get copied to
    REG_DMA0CNT = WordCount | mode; //Set the mode
}

void inline DMACopyCH1(void* source, void* dest, u32 WordCount, u32 mode) {
    REG_DMA1SAD = (u32)source;
    REG_DMA1DAD = (u32)dest;
    REG_DMA1CNT = WordCount | mode;
}

void inline DMACopyCH2(void* source, void* dest, u32 WordCount, u32 mode) {
    REG_DMA2SAD = (u32)source;
    REG_DMA2DAD = (u32)dest;
    REG_DMA2CNT = WordCount | mode;
}

void inline DMACopyCH3(void* source, void* dest, u32 WordCount, u32 mode) {
    REG_DMA3SAD = (u32)source;
    REG_DMA3DAD = (u32)dest;
    REG_DMA3CNT = WordCount | mode;
}

//these functions are very handy for cleaning ploted text
//since the source is fixed, the function will always copy 0
void inline DMAClearMemory16(void* dest, u32 WordCount) {
    REG_DMA0SAD = (u32)&clear; //get the address of "clear" and pass it to the DMA controller as our source
    REG_DMA0DAD = (u32)dest;
    REG_DMA0CNT = WordCount | DMA_ENABLE | DMA_TIMEING_IMMEDIATE | DMA_16 | DMA_SOURCE_FIXED;
}

void inline DMAClearMemory32(void* dest, u32 WordCount) {
    REG_DMA0SAD = (u32)&clear;
    REG_DMA0DAD = (u32)dest;
    REG_DMA0CNT = WordCount | DMA_ENABLE | DMA_TIMEING_IMMEDIATE | DMA_32 | DMA_SOURCE_FIXED;
}

