#ifndef GBA_H
#define GBA_H

typedef unsigned char  u8;
typedef unsigned short u16;
typedef unsigned long  u32;

typedef signed char  s8;
typedef signed short s16;
typedef signed long  s32;

#define VIDEO_BG0_ENABLED 0x100
#define VIDEO_BG1_ENABLED 0x200
#define VIDEO_BG2_ENABLED 0x400
#define VIDEO_BG3_ENABLED 0x800
#define VIDEO_MODE0 0x0
#define VIDEO_MODE1 0x1
#define VIDEO_MODE2 0x2
#define VIDEO_MODE3 0x3
#define VIDEO_MODE4 0x4
#define VIDEO_MODE5 0x5


#define REG_DISP_CNT             (*(u16*)0x4000000)
#define REG_BG0CNT               (*(u16*)0x4000008)
#define REG_BG1CNT               (*(u16*)0x400000A)
#define REG_BG2CNT               (*(u16*)0x400000C)
#define CharBaseBlock(n)		 (((n)*0x4000)+0x6000000)
#define ScreenBaseBlock(n)		 (((n)*0x800)+0x6000000)

#define BG_PRIORITE_0 0x0000
#define BG_PRIORITE_1 0x0001
#define BG_PRIORITE_2 0x0002
#define BG_PRIORITE_3 0x0003
#define BG_CHARBLOCK_0 0x0000
#define BG_CHARBLOCK_1 0x0004
#define BG_CHARBLOCK_2 0x0008
#define BG_CHARBLOCK_3 0x000C
#define BG_EFFET_MOSAIQUE 0x0040
#define BG_1_PALETTE 128
#define BG_16_PALETTES 0x0000
#define BG_WRAP 0x2000
#define BG_TEXTE_256x256 0x0000
#define BG_TEXTE_512x256 0x4000
#define BG_TEXTE_256x512 0x8000
#define BG_TEXTE_512x512 0xC000
#define BG_ROTATION_128x128 0x0000
#define BG_ROTATION_256x256 0x4000
#define BG_ROTATION_512x512 0x8000
#define BG_ROTATION_1024x1024 0xC000

#define MODE_0 0
#define MODE_1 1
#define MODE_2 2
#define MODE_3 3
#define MODE_4 4

#define REG_BGSCROLLX(n) (*(u16*)(0x04000000+0x10+((n)<<2)))
#define REG_BGSCROLLY(n) (*(u16*)(0x04000000+0x12+((n)<<2)))
#define REG_VCOUNT		*(volatile u16*)0x4000006




//DMA registers
#define REG_DMA0SAD     *(u32*)0x40000B0
#define REG_DMA0SAD_L   *(u16*)0x40000B0
#define REG_DMA0SAD_H   *(u16*)0x40000B2
#define REG_DMA0DAD     *(u32*)0x40000B4
#define REG_DMA0DAD_L   *(u16*)0x40000B4
#define REG_DMA0DAD_H   *(u16*)0x40000B6
#define REG_DMA0CNT     *(u32*)0x40000B8
#define REG_DMA0CNT_L   *(u16*)0x40000B8
#define REG_DMA0CNT_H   *(u16*)0x40000BA
#define REG_DMA1SAD     *(u32*)0x40000BC
#define REG_DMA1SAD_L   *(u16*)0x40000BC
#define REG_DMA1SAD_H   *(u16*)0x40000BE
#define REG_DMA1DAD     *(u32*)0x40000C0
#define REG_DMA1DAD_L   *(u16*)0x40000C0
#define REG_DMA1DAD_H   *(u16*)0x40000C2
#define REG_DMA1CNT     *(u32*)0x40000C4
#define REG_DMA1CNT_L   *(u16*)0x40000C4
#define REG_DMA1CNT_H   *(u16*)0x40000C6
#define REG_DMA2SAD     *(u32*)0x40000C8
#define REG_DMA2SAD_L   *(u16*)0x40000C8
#define REG_DMA2SAD_H   *(u16*)0x40000CA
#define REG_DMA2DAD     *(u32*)0x40000CC
#define REG_DMA2DAD_L   *(u16*)0x40000CC
#define REG_DMA2DAD_H   *(u16*)0x40000CE
#define REG_DMA2CNT     *(u32*)0x40000D0
#define REG_DMA2CNT_L   *(u16*)0x40000D0
#define REG_DMA2CNT_H   *(u16*)0x40000D2
#define REG_DMA3SAD     *(u32*)0x40000D4
#define REG_DMA3SAD_L   *(u16*)0x40000D4
#define REG_DMA3SAD_H   *(u16*)0x40000D6
#define REG_DMA3DAD     *(u32*)0x40000D8
#define REG_DMA3DAD_L   *(u16*)0x40000D8
#define REG_DMA3DAD_H   *(u16*)0x40000DA
#define REG_DMA3CNT     *(u32*)0x40000DC
#define REG_DMA3CNT_L   *(u16*)0x40000DC
#define REG_DMA3CNT_H   *(u16*)0x40000DE

#define DMA_ENABLE					0x80000000
#define DMA_INTERUPT_ENABLE			0x40000000
#define DMA_TIMEING_IMMEDIATE		0x00000000
#define DMA_TIMEING_VBLANK			0x10000000
#define DMA_TIMEING_HBLANK			0x20000000
#define DMA_TIMEING_SYNC_TO_DISPLAY 0x30000000
#define DMA_16						0x00000000
#define DMA_32						0x04000000
#define DMA_REPEATE					0x02000000
#define DMA_SOURCE_INCREMENT		0x00000000
#define DMA_SOURCE_DECREMENT		0x00800000
#define DMA_SOURCE_FIXED			0x01000000
#define DMA_DEST_INCREMENT			0x00000000
#define DMA_DEST_DECREMENT			0x00200000
#define DMA_DEST_FIXED				0x00400000
#define DMA_DEST_RELOAD				0x00600000

//these defines group common options to save typing. You may notice that I don't have to include the option to increment the source and address register as that is the default.
#define DMA_32NOW					DMA_ENABLE | DMA_TIMEING_IMMEDIATE |DMA_32 
#define DMA_16NOW					DMA_ENABLE | DMA_TIMEING_IMMEDIATE |DMA_16 

//just an little bit of empty memory
extern u32 clear;

extern void inline DMACopyCH0(void* source, void* dest, u32 WordCount, u32 mode);

extern void inline DMACopyCH1(void* source, void* dest, u32 WordCount, u32 mode);

extern void inline DMACopyCH2(void* source, void* dest, u32 WordCount, u32 mode);

extern void inline DMACopyCH3(void* source, void* dest, u32 WordCount, u32 mode);

//these functions are very handy for cleaning ploted text
//since the source is fixed, the function will always copy 0
extern void inline DMAClearMemory16(void* dest, u32 WordCount);

extern void inline DMAClearMemory32(void* dest, u32 WordCount);

#define RGB(r,g,b) ((r) | (g << 5) | (b << 10))


#define OBJ_ENABLED 0x1000

//attribute0 #defines
#define ROTATION_FLAG		0x100
#define SIZE_DOUBLE		0x200
#define MODE_NORMAL		0x0
#define MODE_TRANSPARENT	0x400
#define MODE_WINDOWED		0x800
#define MOSAIC			0x1000
#define COLOR_16		0x0000
#define COLOR_256		0x2000
#define SQUARE			0x0
#define TALL			0x4000
#define WIDE			0x8000

//attribute1 #defines
#define ROTDATA(n)		((n)<<9)
#define HORIZONTAL_FLIP		0x1000
#define VERTICAL_FLIP		0x2000
#define SIZE_8			0x0
#define SIZE_16			0x4000
#define SIZE_32			0x8000
#define SIZE_64			0xC000

//atrribute2 #defines
#define PRIORITY(n)	        ((n)<<10)
#define PALETTE(n)		((n)<<12)

#define OBJ_MAP_2D 	0x0
#define OBJ_MAP_1D 	0x40

typedef struct {
unsigned short attr0;
unsigned short attr1;
unsigned short attr2;
unsigned short attr3;
} TypeSprite;


typedef struct {
   u8 A, B, L, R, Up, Down, Right, Left, Start, Select, Anykey;
} tPad;

typedef struct {
   tPad Held, Released, Newpress; // Pressed si on appuye, Held si on garde appuy, et Released si on relache
} Pads;

#define BUTTON_A 1
#define BUTTON_B 2
#define BUTTON_SELECT 4
#define BUTTON_START 8
#define BUTTON_RIGHT 16
#define BUTTON_LEFT 32
#define BUTTON_UP 64
#define BUTTON_DOWN 128
#define BUTTON_R 256
#define BUTTON_L 512

#define BUTTONS (*(volatile u16*)0x04000130)

#define UPDATEPAD(type, pad)  type.A = pad & BUTTON_A;\
   type.B = (pad & BUTTON_B) >> 1;\
   type.Select = (pad & BUTTON_SELECT) >> 2;\
   type.Start = (pad & BUTTON_START) >> 3;\
   type.Right = (pad & BUTTON_RIGHT) >> 4;\
   type.Left = (pad & BUTTON_LEFT) >> 5;\
   type.Up = (pad & BUTTON_UP) >> 6;\
   type.Down = (pad & BUTTON_DOWN) >> 7;\
   type.R = (pad & BUTTON_R) >> 8;\
   type.L = (pad & BUTTON_L) >> 9;\
   type.Anykey = (!(!((pad&2047))));





#endif
